#ifndef INDEX_H
#define INDEX_H
// $Id: index.h,v 1.1.1.1 2002/03/24 12:37:01 dvermeir Exp $
#include <map>
#include "stringpool.h"

/**
 * An index structure, mapping words to sets of filenames.
 * In addition, an index keeps a set of words to be ignored and
 * it remembers the time of the last index operation on a filename.
 *
 * Note that the class does not depend on files, only on streams.
 * This is useful as we could e.g. reuse it, replacing filenames with
 * url's.
 */
class Index {
public:
  typedef set<string> Strings;
  typedef set<const string*> Files;
  typedef map<string, Files > Map;
  typedef map<const string*, time_t > Dates;
  typedef Map::const_iterator iterator;

  /**
   * Constructor.
   * \param word_size max size of word.
   * \param ignore stream containing words to be ignored,
   *  0 if no.
   */
  Index(size_t word_size, istream* is=0);
  ~Index();

  /**
   * Return pool reference for filename or 0 if not in pool.
   * \param fn filename to find reference to.
   * \return pointer to internal filename or 0 if unknown.
   */
  const string* file(const string& fn) const { return pool_(fn); }
  /**
   * Return insert date for file.
   * \param fn filename to get insert date for.
   * \return number of seconds since 1/1/1970 until time of last index
   *   operation for filename or 0 is filenams is unknown.
   */
  time_t date(const string& fn) const;

  /**
   * Consult index.
   * \param s word to look for.
   * \return set of (pointers to) filenames or 0.
   */
  const Files* query(const string&) const;

  /**
   * Consult index.
   * \param q set of words to look for
   * \param r will contain result, set of pointers to filenames of
   *   files containing all words from q.
   * \return reference to second parameter.
   */
  Files& query(const Strings& q, Files& r) const;

  /**
   * Insert a range of words from filename into the index.
   * \param fn filename that will be associated with each word
   * \param is stream from which words will be inserted
   * \return count of words actually inserted
   */
  size_t insert(const string& fn, istream& is);

  const Strings& ignore() const { return ignore_; }
  /**
   * Put a word on the ignore list.
   * \param w word to be put on the ignore list.
   */
  void ignore(const string& w);
  /**
   * Put all words from stream on the ignore list.
   * \param is stream from which words are to be put on the ignore list.
   */
  void ignore(istream& is);
  /**
   * Remove all words associated with filename from the index.
   * \param fn filename to be erased from the index.
   */
  void remove(const string& fn);

  /**
   * Write the index to a text file.
   *
   * Format:
   * \code
   * #textindexer VERSION -- do not edit this file
   * time_t filename
   * ...
   * empty line
   * word fileid ..
   * \endcode
   * where \a fileid is an id referring to the stringpool
   */
  friend ostream&
    operator<<(ostream& os, const Index& index);
  /**
   * Read the index from a text file.
   */
  friend istream&
    operator>>(istream& os, Index& index) throw (runtime_error);

  StringPool& pool() { return pool_; }
  const StringPool& pool() const { return pool_; }

  iterator begin() const { return map_.begin(); }
  iterator end() const { return map_.end(); }
  size_t word_size() const { return word_size_; }
  size_t size() const { return map_.size(); }
  /**
   * \example test-index.C
   */

private:
  Strings ignore_;
  StringPool pool_;
  Map map_;
  Dates dates_;
  size_t word_size_;
};
#endif

