#ifndef QUERY_H
#define QUERY_H
// $Id: query.h,v 1.1.1.1 2002/03/24 12:37:02 dvermeir Exp $
#include <string>
#include <list>
#include "index.h"

/**
 * An query object represents a query on a particular index structure.
 * The syntax of a query, which is parsed by the constructor, is
 * \code
 * query := [ word | phrase ]*
 * phrase := " word * "
 * word := string
 * \endcode
 */
class Query {
public:
  typedef list<string> Phrase;
  /**
   * Constructor.
   * \param index Index structure on which query will be run.
   * \param q string containing the query text.
   * \exception runtime_error if mallformed (incl. empty, after
   *   removing ignored words) query.
   */
  Query(const Index& index, const string& q) throw (runtime_error);
  ~Query() {}
  
  /**
   * Execute a query. A file known by the indexer of the query
   * satisfies the query if 
   * - it contains each single word in the query
   * - it contains all phrases in the query.
   *
   * A file contains a phrase "a b c" if, when viewed as a sequence of
   * words (a word is a maximal string of alphabetic characters)
   * it contains a sequence of
   * words " a j1 b j2 c" where \a j1 and \a j2 are sequences of
   * ignored words (w.r.t. the query's index).
   * \return a reference to a set of filenames that satisfy the query.
   * The result of the query can be retrieved again using Query::exec.
   */
  const Index::Files& exec();

  /**
   * \return stored result of Query::exec.
   * \warning
   *   This function assumes that Query::exec has been called before.
   */
  const Index::Files& result() const { return result_; }
  friend ostream&
    operator<<(ostream& os, const Query& q);
private:
  const Index& index_;
  string query_;
  list<Phrase> phrases_;
  Index::Strings words_;
  Index::Files result_;
};
#endif

