#ifndef SERVER_H
#define SERVER_H
// $Id: server.h,v 1.1.1.1 2002/03/24 12:37:02 dvermeir Exp $
#include <string>
#include <stdexcept>
#include <iostream>

#include "index.h"

/**
 * The server class.
 * Note that it does not use any network classes or functions.
 * The core member function is serve(iostream&) which handles a single
 * connection.
 */
class Server {
public:
  /**
   * Constructor.
   * The configuration should contain the following parameters:
   * \code
   * portnr	-- not interpreted, usually port# of serversocket
   * index	-- filename of index (this file need not exist)
   * wordlength -- size of words used in the index
   * ignore     -- filename containing words to be ignored
   * logfile    -- name of logfile
   * maxcmd	-- maximal size (in chars) of a commandline
   *		   (optional, default is 2048).
   * \endcode
   * \param config_fn pathname of server configuration file.
   * \exception runtime_error is some parameters are missing from the 
   *   configuration file, some file cannot be opened etc.
   */
  Server(const string& config_fn) throw (runtime_error);
  ~Server();

  /**
   * Does path refer to a file suitable for indexing.
   * \param path to check.
   * \param err stream to write error messages to
   * \return true iff the following conditions hold:
   * - File f(path) succeeds and f.exists() returns true.
   * - f.type() == File::REGULAR
   * - ifstream ifs(path) succeeds.
   * - the first 100 chars of the file are printable or space ('\n' etc.).
   */
  bool ok_to_index(const string& path, ostream& err) const;
  /**
   * Perform a single session with an iostream (usually a Socket) ios.
   * \return true iff "stop" command was not received.
   */
  bool serve(iostream& ios);

  const string& config_fn() const { return config_fn_; }
  const string& index_fn() const { return index_fn_; }
  const string& ignore_fn() const { return ignore_fn_; }
  const string& log_fn() const { return log_fn_; }
  const int port() const { return port_; }
  const string::size_type wordlength() const { return wordlength_; }
  const string::size_type max_cmd() const { return max_cmd_; } 
  const string error() const { return "ERROR "; }
  const string ok() const { return "OK"; }
  ostream& log() const { return *log_; }
  const Index& index() const { return *index_; }
  Index& index() { return *index_; }

private:
  string config_fn_;
  string index_fn_;
  string ignore_fn_;
  string log_fn_;
  int port_;
  string::size_type wordlength_;
  string::size_type max_cmd_;

  ostream* log_;
  Index* index_;
};
#endif

