#ifndef STRINGPOOL_H
#define STRINGPOOL_H
// $Id: stringpool.h,v 1.1.1.1 2002/03/24 12:37:03 dvermeir Exp $
#include <string>
#include <set>
#include <stdexcept>

/**
 * A StringPool allows the sharing of identical strings.
 * Also, comparison can be done on the pointers (much faster).
 *
 * \code
 * StringPool pool;
 * string* s = pool["abc"];
 * if (t = pool("abc"))
 *   cout << *t << " already in pool";
 * \endcode
 */
class StringPool {
public:
  /** Constructor */
  StringPool();
  /** Destructor */
  ~StringPool();
  /**
   * Try to retrieve a string from the pool.
   * \param s string to look for.
   * \return pointer to the pool's copy or 0 if not found
   * \warning Never delete a pointer returned by this function.
   */
  const string* operator()(const string& s) const;
  /**
   * Try to retrieve a string from the pool, add it
   * (a copy will be made) if not found.
   * \param s string to find/add.
   * \return pointer to the pool's copy os \a a.
   * \warning Never delete a pointer returned by this function.
   */
  const string* operator[](const string& s);
  /**
   * Comparison function that compares the dereferences pointers.
   * \return \a (*s1<*s2)
   * \exception runtime_error if any of the arguments is null.
   */
  struct compare {
    bool operator()(const string* s1, const string* s2) const throw (runtime_error) ;
    };
  /**
   * Remove a string from the stringpool. 
   * \param s string to remove.
   * \warning
   *   Only use when you are certain there are no references to this
   *   string. Does not complain if the string is not in the pool.
   */
  void remove(const string& s);
  typedef set<const string*, compare> PtrSet;
  typedef PtrSet::const_iterator iterator;

  iterator begin() const { return pool_.begin(); }
  iterator end() const { return pool_.end(); }
  size_t size() const { return pool_.size(); }
  /**
   * \example test-stringpool.C
   */
private:
  PtrSet pool_;
};
#endif

